#include "Database.h"

Int16 Database::Compare(void* rec1, void* rec2, Int16 other,
						SortRecordInfoPtr rec1SortInfo,
						SortRecordInfoPtr rec2SortInfo,
						MemHandle appInfoH) {
   Record r1((MemPtr)rec1);
   Record r2((MemPtr)rec2);

   UInt32 s1 = ::TimDateTimeToSeconds(r1.getDateTime());
   UInt32 s2 = ::TimDateTimeToSeconds(r2.getDateTime());
   Int16 result = 0;
   if (s1 < s2)
	  result = -1;
   if (s1 > s2)
	  result = 1;
   if (other == SORT_BY_DATE_DESC)
	  return -result;
   return result;
}

Err Database::open(UInt16 mode) {
   Boolean created;
   Err err = Dm::OpenOrCreateDatabase(&_db, DB_TYPE, CREATOR, mode, 0, DB_NAME, &created);
   if (created) {
	  MemHandle h = ::DmNewHandle(_db, sizeof(AppInfoType));
	  if (!h)
		 return dmErrMemError;
	  AppInfoType *app = (AppInfoType *)::MemHandleLock(h);
	  ::DmSet(app, 0, sizeof(AppInfoType), 0);
	  ::CategoryInitialize((AppInfoType *)app, IdDefaultCategories);
	  ::MemHandleUnlock(h);
	  Dm::SetAppInfo(_db, h);
   }
   _sortType = SORT_BY_DATE_ASC;
   _isArchive = true;
   _isNew = false;
   _index = NO_RECORD_SELECTED;
   _category = dmAllCategories;
   return err;
}

void Database::attachRecord(Record &r, MemHandle h) {
   r.pack(::MemHandleLock(h));
   ::MemHandleUnlock(h);
   _index = Dm::InsertRecord(_db, h, Database::Compare, _sortType);
   Dm::SetRecordCategory(_db, _index, _category);
}

UInt16 Database::createRecord() {
   Record r;
   MemHandle h = ::DmNewHandle(_db, r.size());
   if (!h){
	  ::FrmAlert(DeviceFullAlert);
	  _index = NO_RECORD_SELECTED;
   } else {
	  attachRecord(r, h);
	  _isNew = true;
   }
   return _index;
}

UInt16 Database::updateRecord(Record &r) {
   MemHandle h;
   ::DmDetachRecord(_db, _index, &h);
   Err err = ::MemHandleResize(h, r.size());
   if (err != 0) {
	  ::FrmAlert(DeviceFullAlert);
	  _index = NO_RECORD_SELECTED;
   } else {
	  attachRecord(r, h);
	  _isNew = false;
   }
   return _index;
}

void Database::deleteRecord() {
   if (_isNew) {
	  ::DmRemoveRecord(_db, _index);
   } else {
	  if (_isArchive) {
		 ::DmArchiveRecord(_db, _index);
	  } else {
		 ::DmDeleteRecord(_db, _index);
	  }
	  ::DmMoveRecord(_db, _index, ::DmNumRecords(_db));
   }
   _index = NO_RECORD_SELECTED;
}

Boolean Database::selectCategory(UInt16 ctlId, UInt16 lstId, Boolean title = true) {
   FormPtr frm = ::FrmGetActiveForm();
   return Dm::SelectCategory(_db, &_category, frm, ctlId, lstId, title);
}
