#ifndef __DATABASE_H__
#define __DATABASE_H__

#include <PalmOS.h>
#include <CatooCraft/DmHelper.h>
#include "Constants.h"
#include "Record.h"

using namespace CatooCraftLibrary;

#define DB_NAME		"Record-Milg"
#define DB_TYPE		'DATA'

class Database
{
   public:
	  static const UInt16 NO_RECORD_SELECTED = (UInt16)-1;
	  typedef enum {
		 SORT_BY_DATE_ASC,
		 SORT_BY_DATE_DESC
	  } SortType;

   private:
	  DmOpenRef _db;
	  SortType _sortType;
	  Boolean _isArchive;
	  UInt16 _category;
	  UInt16 _index;
	  Boolean _isNew;

	  static Int16 Compare(void* rec1, void* rec2, Int16 other,
						   SortRecordInfoPtr rec1SortInfo,
						   SortRecordInfoPtr rec2SortInfo,
						   MemHandle appInfoH);
	  void attachRecord(Record &r, MemHandle h);

   public:
	  void setSortType(SortType sortType) {
		 _sortType = sortType;
	  }
	  SortType getSortType() {
		 return _sortType;
	  }
	  void setArchiveFlag(Boolean isArchive) {
		 _isArchive = isArchive;
	  }
	  Boolean isArchive() {
		 return _isArchive;
	  }
	  Boolean isNew() {
		 return _isNew;
	  }
	  Err open(UInt16 mode);
	  void close() {
		 ::DmCloseDatabase(_db);
	  }
	  void changeCategory(UInt16 category) {
		 _category = category;
	  }
	  UInt16 getCategory() {
		 return _category;
	  }
	  UInt16 getRecordCategory(UInt16 index) {
		 return Dm::GetRecordCategory(_db, index);
	  }
	  UInt16 numRecords() {
		 return ::DmNumRecordsInCategory(_db, _category);
	  }
	  UInt16 createRecord();
	  UInt16 updateRecord(Record &r);
	  void deleteRecord();
	  Err getUniqueId(UInt16 index, UInt32 *uniqueId) {
		 return ::DmRecordInfo(_db, index, NULL, uniqueId, NULL);
	  }
	  Err findRecordByID(UInt32 uniqueId, UInt16 *index) {
		 return ::DmFindRecordByID(_db, uniqueId, index);
	  }
	  Err seekRecord(UInt16 *index, Int16 offset, Int16 direction) {
		 return ::DmSeekRecordInCategory(_db, index, offset, direction, _category);
	  }
	  MemHandle queryRecord(UInt16 index) {
		 return ::DmQueryRecord(_db, index);
	  }
	  MemHandle queryNext(UInt16 *index) {
		 return ::DmQueryNextInCategory(_db, index, _category);
	  }
	  void quickSort() {
		 ::DmQuickSort(_db, Database::Compare, _sortType);
	  }
	  void markRecord(UInt16 index) {
		 _isNew = false;
		 _index = index;
	  }
	  UInt16 getMarkedRecordIndex() {
		 return _index;
	  }
	  
	  const Char* getCategoryName() {
		 return Dm::CategoryGetName(_db, _category);
	  }
	  Boolean selectCategory(UInt16 ctlId, UInt16 lstId, Boolean title = true);
};

#endif __DATABASE_H__
