#include <TestSuite.h>
#include <TestCaller.h>
#include "DatabaseTest.h"
#include "Record.h"

Test* DatabaseTest::suite() {
    TestSuite* suite = new TestSuite("DatabaseTest");
	suite->addTest(new TestCaller<DatabaseTest>("setter getter", &DatabaseTest::testSetterGetter));
	suite->addTest(new TestCaller<DatabaseTest>("open", &DatabaseTest::testOpen));
	suite->addTest(new TestCaller<DatabaseTest>("create", &DatabaseTest::testCreateRecord));
	suite->addTest(new TestCaller<DatabaseTest>("update", &DatabaseTest::testUpdateRecord));
	suite->addTest(new TestCaller<DatabaseTest>("delete", &DatabaseTest::testDeleteRecord));

    return suite;
}

void DatabaseTest::setUp() {
   LocalID dbID = ::DmFindDatabase(0, DB_NAME);
   if (dbID != 0) {
	  Err err = ::DmDeleteDatabase(0, dbID);
	  ErrFatalDisplayIf(err != 0, "Cannot Delete Database");
   }
}

void DatabaseTest::testSetterGetter() {
   Database db;
   db.open(dmModeReadWrite);

   db.setArchiveFlag(false);
   AssertEquals(false, db.isArchive());
   db.setArchiveFlag(true);
   AssertEquals(true, db.isArchive());

   db.changeCategory(0);
   AssertEquals(0, db.getCategory());
   db.changeCategory(dmAllCategories);
   AssertEquals(dmAllCategories, db.getCategory());

   db.close();
}

void DatabaseTest::testOpen() {
   Database db;
   db.open(dmModeReadWrite);
   db.close();
   db.open(dmModeReadWrite);
   db.close();
}

void DatabaseTest::testCreateRecord() {
   Database db;
   db.open(dmModeReadWrite);

   AssertEquals(0, db.numRecords());
   UInt16 index = db.createRecord();
   AssertEquals(1, db.numRecords());
   MemHandle h = db.queryRecord(index);
   Record r(::MemHandleLock(h));
   AssertEquals("", r.getNote());
   ::MemHandleUnlock(h);

   AssertEquals(1, db.numRecords());
   db.changeCategory(1);
   AssertEquals(0, db.numRecords());
   index = db.createRecord();
   AssertEquals(1, db.numRecords());

   db.changeCategory(dmAllCategories);
   AssertEquals(2, db.numRecords());

   db.close();
}

void DatabaseTest::testUpdateRecord() {
   Database db;
   db.open(dmModeReadWrite);

   UInt16 index = db.createRecord();
   MemHandle h = db.queryRecord(index);
   Record r;
   r.copy(::MemHandleLock(h));
   ::MemHandleUnlock(h);
   AssertEquals("", r.getNote());
   r.setNote("update test");
   index = db.updateRecord(r);
   h = db.queryRecord(index);
   Record rr(::MemHandleLock(h));
   AssertEquals("update test", rr.getNote());
   ::MemHandleUnlock(h);

   db.close();
}

void DatabaseTest::testDeleteRecord() {
   Database db;
   db.open(dmModeReadWrite);

   UInt16 index = db.createRecord();
   AssertEquals(1, db.numRecords());
   db.deleteRecord();
   AssertEquals(0, db.numRecords());

   index = db.createRecord();
   Record r;
   db.updateRecord(r);
   AssertEquals(1, db.numRecords());
   db.deleteRecord();
   AssertEquals(0, db.numRecords());

   db.close();
}
