#include <PalmOS.h>
#include <CatooCraft/FrmHelper.h>
#include <CatooCraft/FldHelper.h>
#include <CatooCraft/CashBookMgr.h>
#include <CatooCraft/NumberFormat.h>
#include "MileageRsc.h"
#include "Mileage.h"
#include "Record.h"
#include "ExportForm.h"

using namespace CatooCraftLibrary;

UInt16 ExportForm::_RefNum;
UInt16 ExportForm::_Category = 0;

Boolean ExportForm::HandleEvent(EventPtr e) {
   switch (e->eType) {
	  case ctlSelectEvent:
		 if (e->data.ctlSelect.controlID == IdCategoryPopTrigger) {
			Char name[dmCategoryLength];
			::CbmTransactionCategorySelect(_RefNum, ::FrmGetActiveForm(), 
										   IdCategoryPopTrigger, IdCategoryList, false,
										   &_Category, name, 0);
			
			return true;
		 }
		 break;

	  case fldChangedEvent:
		 Fld::UpdateScrollbar(::FrmGetActiveForm(), IdNoteField, IdNoteFieldScrollbar);
		 return true;

	  case sclRepeatEvent:
		 Fld::ScrollLines(IdNoteField, IdNoteFieldScrollbar,
						  e->data.sclRepeat.newValue - e->data.sclRepeat.value, false);
		 break;

	  case keyDownEvent:
		 if (e->data.keyDown.chr == pageUpChr) {
			Fld::PageScroll(IdNoteField, IdNoteFieldScrollbar, winUp);
			return true;
		 } else if (e->data.keyDown.chr == pageDownChr) {
			Fld::PageScroll(IdNoteField, IdNoteFieldScrollbar, winDown);
			return true;
		 }
		 break;

	  default:
		 break;
   }
   return false;
}

Err ExportForm::OpenLibrary() {
   Err err;
   err = ::SysLibFind(CashBookMgrName, &_RefNum);
   if (err)
	  err = ::SysLibLoad(CashBookLibType, CashBookMgrCreator, &_RefNum);

   if (err)
	  return err;

   return ::CbmOpen(_RefNum);
}

void ExportForm::CloseLibrary() {
   Err err = ::CbmClose(_RefNum);
   if (!err) {
	  ::SysLibRemove(_RefNum);
   }
}

UInt16 ExportForm::DoModal(Record &r) {
   Err err;

   err = OpenLibrary();
   if (err) {
	  ::FrmAlert(IdNoCashBookMgrAlert);
	  return IdCancelButton;
   }

   Boolean isPresent;
   err = ::CbmIsPresent(_RefNum, &isPresent);
   if (err || !isPresent) {
	  ::FrmAlert(IdNoCashBookAlert);
	  return IdCancelButton;
   }

   FormPtr frm = ::FrmInitForm(IdExportToCashBookForm);
   ::FrmSetEventHandler(frm, ExportForm::HandleEvent);

   Char category[dmCategoryLength];
   Char buff[32];

   // Setup category popup;
   ::CbmTransactionCategoryGetName(_RefNum, _Category, category);
   ::CtlSetLabel(Frm::GetObjectPtr(frm, IdCategoryPopTrigger), category);

   // Setup date
   SystemPreferencesType sysPrefs;
   ::PrefGetPreferences(&sysPrefs);
   DateTimeType *dtp = r.getDateTime();
   ::DateToAscii(dtp->month, dtp->day, dtp->year, sysPrefs.dateFormat, buff);
   Frm::SetText(frm, IdDateField, buff);

   // Setup From and To popup list;
   MemHandle accounts;
   Int16 count;
   MemHandle otherStringH = ::DmGetResource(strRsc, IdOtherString);
   ::CbmGetAccountListStrings(_RefNum, &accounts, &count, (char *)::MemHandleLock(otherStringH));
   ::MemHandleUnlock(otherStringH);
   ::DmReleaseResource(otherStringH);
   MemHandle strings = SysFormPointerArrayToStrings((Char *)::MemHandleLock(accounts), count);
   Char **choices = (Char **)::MemHandleLock(strings);
   ListPtr from = (ListType *)Frm::GetObjectPtr(frm, IdFromList);
   ListPtr to = (ListType *)Frm::GetObjectPtr(frm, IdToList);
   LstSetListChoices(from, choices, count);
   LstSetListChoices(to, choices, count);
   LstSetHeight(from, (count > 10 ? 10 : count));
   LstSetHeight(to, (count > 10 ? 10 : count));
   Frm::SetupPopupList(frm, IdFromPopTrigger, IdFromList, count-1);
   Frm::SetupPopupList(frm, IdToPopTrigger, IdToList, count-1);

   // Setup amount
   Frm::SetText(frm, IdAmountField, NumberFormat::Format(buff, r.getCost()));

   // Setup memo
   Frm::SetText(frm, IdNoteField, r.getVendor());

   UInt16 hitButton = ::FrmDoDialog(frm);

   if (hitButton == IdOKButton) {
	  CbmTransactionType t;
	  t.dateTime = *(r.getDateTime());
	  t.amount = r.getCost();
	  t.from = ::LstGetSelectionText(from, ::LstGetSelection(from));
	  t.to = ::LstGetSelectionText(to, ::LstGetSelection(to));
	  t.note = Frm::GetText(frm, IdNoteField, (Char *)::MemPtrNew(2048+1));
	  t.receipt = NULL;
	  ::CbmAdjustTransaction(_RefNum, &t, _Category);
	  ::MemPtrFree(t.note);
   }

   ::FrmDeleteForm(frm);
   ::MemHandleUnlock(accounts);
   ::MemHandleFree(accounts);
   ::MemHandleUnlock(strings);
   ::MemHandleFree(strings);

   CloseLibrary();
   return hitButton;
}
