#include <PalmOS.h>
#include <CatooCraft/NumberFormat.h>
#include "Constants.h"
#include "Mileage.h"
#include "MileageRsc.h"
#include "RecordForm.h"
#include "RecordListForm.h"
#include "PrefsForm.h"

Database Mileage::_Db;
VendorHistory Mileage::_History;

typedef struct {
	  UInt16 lastCategory;
	  Boolean saveBackup;
	  Boolean isAutoComplete;
	  Database::SortType sortType;
	  UInt8 decimalPlaces;
	  Record::DistanceUnitType distanceUnitType;
	  Record::AmountUnitType amountUnitType;
	  Record::CalculationType calculationType;
	  Int16 numHistories;
	  Char history[0];
} MileagePreferenceType;

void Mileage::Start(void) {
   _Db.open(dmModeReadWrite);

   UInt16 prefsSize = 0;
   if (::PrefGetAppPreferences(CREATOR, PREFS_ID, NULL, &prefsSize, true) != noPreferenceFound) {
	  MileagePreferenceType *prefs = (MileagePreferenceType *)::MemPtrNew(prefsSize);
	  ::PrefGetAppPreferences(CREATOR, PREFS_ID, prefs, &prefsSize, true);
	  _Db.changeCategory(prefs->lastCategory);
	  _Db.setArchiveFlag(prefs->saveBackup);
	  _Db.setSortType(prefs->sortType);
	  VendorHistory::SetAutoCompleteFlag(prefs->isAutoComplete);
	  NumberFormat::SetDecimalPlaces(prefs->decimalPlaces);
	  Record::SetDistanceUnitType(prefs->distanceUnitType);
	  Record::SetAmountUnitType(prefs->amountUnitType);
	  Record::SetCalculationType(prefs->calculationType);
	  _History.setup(&prefs->history[0], prefs->numHistories);
	  ::MemPtrFree(prefs);
   }
   ::FrmGotoForm(IdRecordListForm);
}

void Mileage::Stop(void) {
   ::FrmSaveAllForms();
   ::FrmCloseAllForms();

   Int16 size = sizeof(MileagePreferenceType) + _History.size();
   MileagePreferenceType *prefs;
   prefs = (MileagePreferenceType *)::MemPtrNew(size);
   prefs->lastCategory = _Db.getCategory();
   prefs->saveBackup = _Db.isArchive();
   prefs->sortType = _Db.getSortType();
   prefs->isAutoComplete = VendorHistory::IsAutoComplete();
   prefs->decimalPlaces = NumberFormat::GetDecimalPlaces();
   prefs->distanceUnitType = Record::GetDistanceUnitType();
   prefs->amountUnitType = Record::GetAmountUnitType();
   prefs->calculationType = Record::GetCalculationType();
   prefs->numHistories = _History.count();
   _History.pack(&prefs->history[0]);
   ::PrefSetAppPreferences(CREATOR, PREFS_ID, PREFS_VER, prefs, size, true);
   ::MemPtrFree(prefs);
   _Db.close();
}

Boolean Mileage::HandleEvent(EventPtr e) {
   FormPtr frm;
   UInt16    formId;
   Boolean handled = false;

   if (e->eType == frmLoadEvent)
   {
	  formId = e->data.frmLoad.formID;
	  frm = ::FrmInitForm(formId);
	  ::FrmSetActiveForm(frm);

	  switch(formId)
	  {
		 case IdRecordListForm:
			::FrmSetEventHandler(frm, RecordListForm::HandleEvent);
			break;

		 case IdRefillForm:
			::FrmSetEventHandler(frm, RecordForm::HandleEvent);
			break;

		 default:
			return false;
	  }
	  handled = true;
   }

   return handled;
}

void Mileage::EventLoop(void) {
   UInt16 err;
   EventType e;

   do {
	  ::EvtGetEvent(&e, evtWaitForever);
	  if (! ::SysHandleEvent(&e))
		 if (! ::MenuHandleEvent(NULL, &e, &err))
			if (! HandleEvent(&e))
			   ::FrmDispatchEvent(&e);
   } while (e.eType != appStopEvent);
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags) {
   if (cmd == sysAppLaunchCmdNormalLaunch) {
	  Mileage::Start();
	  Mileage::EventLoop();
	  Mileage::Stop();
   } else {
	  return sysErrParamErr;
   }
   return 0;
}
