#include <CatooCraft/NumberFormat.h>
#include "Record.h"

using namespace CatooCraftLibrary;

Record::DistanceUnitType Record::_DistanceUnitType;
Record::AmountUnitType Record::_AmountUnitType;
Record::CalculationType Record::_CalculationType;

Record::Record() {
   _unpacked = false;
   _data = (DataType *)::MemPtrNew(sizeof(DataType));
   ::MemSet(_data, sizeof(DataType), 0);
   ::TimSecondsToDateTime(::TimGetSeconds(), &_data->dateTime);
   _data->distanceUnitType = _DistanceUnitType;
   _data->amountUnitType = _AmountUnitType;
   _data->exported = false;
   _vendor = (Char *)::MemPtrNew(1);
   _note = (Char *)::MemPtrNew(1);
   _vendor[0] = _note[0] = '\0';
}

Record::Record(MemPtr p) {
   _unpacked = true;
   _data = (DataType *)p;
   (Char *)p += sizeof(DataType);
   _vendor = (Char *)p;
   (Char *)p += ::StrLen(_vendor) + 1;
   _note = (Char *)p;
}

Record::~Record() {
   if (!_unpacked) {
	  ::MemPtrFree(_data);
	  ::MemPtrFree(_vendor);
	  ::MemPtrFree(_note);
   }
}

void Record::setVendor(Char *vendor) {
   ::MemPtrFree(_vendor);
   _vendor = (Char *)::MemPtrNew(::StrLen(vendor) + 1);
   ::StrCopy(_vendor, vendor);
}

void Record::setNote(Char *note) {
   ::MemPtrFree(_note);
   _note = (Char *)::MemPtrNew(::StrLen(note) + 1);
   ::StrCopy(_note, note);
}

Int16 Record::size() {
   return
	  sizeof(DataType) +
	  ::StrLen(_vendor) + 1 +
	  ::StrLen(_note) + 1;
}

void Record::pack(MemPtr p) {
   UInt16 offset = 0;
   ::DmWrite(p, offset, _data, sizeof(DataType));
   offset += sizeof(DataType);
   ::DmStrCopy(p, offset, _vendor);
   offset += StrLen(_vendor) + 1;
   ::DmStrCopy(p, offset, _note);
}

void Record::copy(MemPtr p) {
   *_data = *(DataType *)p;
   (Char *)p += sizeof(DataType);
   setVendor((Char *)p);
   (Char *)p += ::StrLen(_vendor) + 1;
   setNote((Char *)p);
}

Char *Record::getCalculation(Char * buff) {
   UInt32 dur[] = {
	  KM_RATE,
	  MILE_RATE
   };
   UInt32 aur[] = {
	  LITER_RATE,
	  GALLON_UK_RATE,
	  GALLON_US_RATE
   };

   if (getAmount() == 0) {
	  ::StrCopy(buff, "-----");
   } else {
	  switch (_CalculationType) {
		 case CALC_DISTANCE:
			NumberFormat::Format(buff,
								 getDistance() * dur[getDistanceUnitType()]
								 / dur[_DistanceUnitType],
								 1);
			break;
		 case CALC_AMOUNT:
			NumberFormat::Format(buff,
								 getAmount() * aur[getAmountUnitType()]
								 / aur[_AmountUnitType],
								 2);
			break;
		 case CALC_MILEAGE:
			NumberFormat::Format(buff,
								 getDistance() * dur[getDistanceUnitType()]
								 / dur[_DistanceUnitType] * 1000L
								 / (getAmount() * aur[getAmountUnitType()]
									/ aur[_AmountUnitType]),
								 2);
			break;
		 case CALC_COST:
			NumberFormat::Format(buff, getCost());
			break;
		 case CALC_PRICE:
			NumberFormat::Format(buff,
								 getCost() * aur[_AmountUnitType] * 100L
								 / getAmount() / aur[getAmountUnitType()]);
			break;
	  }
   }
   return buff;
}
