#ifndef __RECORD_H__
#define __RECORD_H__

#include <PalmOS.h>

class Record
{
   public:
	  typedef enum {
		 REGULAR,
		 PLUS,
		 PREMIUM,
		 OTHER
	  } GradeType;

	  typedef enum {
		 KM,
		 MILE
	  } DistanceUnitType;

	  typedef enum {
		 LITER,
		 GALLON_UK,
		 GALLON_US
	  } AmountUnitType;

	  typedef enum {
		 CALC_DISTANCE,
		 CALC_AMOUNT,
		 CALC_MILEAGE,
		 CALC_COST,
		 CALC_PRICE
	  } CalculationType;

   private:
	  typedef enum {
		 LITER_RATE = 100,
		 GALLON_UK_RATE = 455,
		 GALLON_US_RATE = 379
	  } AmountUnitRateType;

	  typedef enum {
		 KM_RATE = 100,
		 MILE_RATE = 161
	  } DistanceUnitRateType;

	  typedef struct {
			DateTimeType dateTime;
			UInt32 distance;
			DistanceUnitType distanceUnitType;
			GradeType grade;
			UInt16 amount;
			AmountUnitType amountUnitType;
			UInt16 cost;
			Boolean exported;
	  } DataType;

	  static DistanceUnitType _DistanceUnitType;
	  static AmountUnitType _AmountUnitType;
	  static CalculationType _CalculationType;

	  Boolean _unpacked;
	  DataType *_data;
	  Char * _vendor;
	  Char * _note;

   public:
	  static void SetDistanceUnitType(DistanceUnitType type) {
		 _DistanceUnitType = type;
	  }
	  static DistanceUnitType GetDistanceUnitType() {
		 return _DistanceUnitType;
	  }
	  static void SetAmountUnitType(AmountUnitType type) {
		 _AmountUnitType = type;
	  }
	  static AmountUnitType GetAmountUnitType() {
		 return _AmountUnitType;
	  }
	  static void SetCalculationType(CalculationType type) {
		 _CalculationType = type;
	  }
	  static CalculationType GetCalculationType() {
		 return _CalculationType;
	  }

	  Record();
	  Record(MemPtr p);
	  ~Record();
	  void setDateTime(DateTimeType *dtp) {
		 _data->dateTime = *dtp;
	  }
	  void setDistance(UInt32 distance) {
		 _data->distance = distance;
	  }
	  void setDistanceUnitType(DistanceUnitType type) {
		 _data->distanceUnitType = type;
	  }
	  void setGrade(GradeType grade) {
		 _data->grade = grade;
	  }
	  void setAmount(UInt16 amount) {
		 _data->amount = amount;
	  }
	  void setAmountUnitType(AmountUnitType amountUnitType) {
		 _data->amountUnitType = amountUnitType;
	  }
	  void setCost(UInt16 cost) {
		 _data->cost = cost;
	  }
	  void setVendor(Char *vendor);
	  void setNote(Char *note);
	  void setExported(Boolean exported) {
		 _data->exported = exported;
	  }
	  DateTimeType *getDateTime() {
		 return &_data->dateTime;
	  }
	  UInt32 getDistance() {
		 return _data->distance;
	  }
	  DistanceUnitType getDistanceUnitType() {
		 return _data->distanceUnitType;
	  }
	  GradeType getGrade() {
		 return _data->grade;
	  }
	  UInt16 getAmount() {
		 return _data->amount;
	  }
	  AmountUnitType getAmountUnitType() {
		 return _data->amountUnitType;
	  }
	  UInt16 getCost() {
		 return _data->cost;
	  }
	  Char *getVendor() {
		 return _vendor;
	  }
	  Char *getNote() {
		 return _note;
	  }
	  Boolean isExported() {
		 return _data->exported;
	  }
	  Char * getCalculation(Char *buff);

	  Int16 size();
	  void pack(MemPtr p);
	  void copy(MemPtr p);
};

#endif __RECORD_H__
