#include <PalmOS.h>
#include <CatooCraft/FrmHelper.h>
#include <CatooCraft/FldHelper.h>
#include <CatooCraft/SclHelper.h>
#include <CatooCraft/Util.h>
#include <CatooCraft/NumberFormat.h>
#include "Database.h"
#include "RecordForm.h"
#include "DeleteRecordForm.h"
#include "ExportForm.h"

using namespace CatooCraftLibrary;

Record RecordForm::_Record;

void RecordForm::DrawListLine(Int16 itemNum, RectangleType *bounds, Char **itemsText) {
   VendorHistory *history = Mileage::GetHistory();
   if (history->count() == 0) {
	  MemHandle h = ::DmGetResource(strRsc, IdHistoryNoneString);
	  Util::DrawCharsToFitWidth((char *)::MemHandleLock(h), bounds);
	  ::MemHandleUnlock(h);
	  ::DmReleaseResource(h);
   } else {
	  Util::DrawCharsToFitWidth((*history)[itemNum], bounds);
   }
}

void RecordForm::DrawDistance(FormPtr frm, UInt32 distance) {
   RectangleType rec;
   MemHandle h;
   Int16 i = 0;
   ::FrmGetObjectBounds(frm, ::FrmGetObjectIndex(frm, IdDistanceButton5 + i), &rec);
   h = ::DmGetResource(bitmapRsc, IdWhite0Bitmap + (distance % (UInt32)10));
   ::WinDrawBitmap((BitmapPtr)::MemHandleLock(h), rec.topLeft.x, rec.topLeft.y);
   ::MemHandleUnlock(h);
   ::DmReleaseResource(h);
   distance /= (UInt32)10;
   for (i = 1; i < 5; i++) {
	  ::FrmGetObjectBounds(frm, ::FrmGetObjectIndex(frm, IdDistanceButton5 + i), &rec);
	  h = ::DmGetResource(bitmapRsc, IdBlack0Bitmap + (distance % (UInt32)10));
	  ::WinDrawBitmap((BitmapPtr)::MemHandleLock(h), rec.topLeft.x, rec.topLeft.y);
	  ::MemHandleUnlock(h);
	  ::DmReleaseResource(h);
	  distance /= (UInt32)10;
   }
}

void RecordForm::UpdateForm() {
   FormPtr frm = ::FrmGetActiveForm();

   Database *db = Mileage::GetDatabase();
   ::CtlSetLabel(Frm::GetObjectPtr(frm, IdCategoryPopTrigger), db->getCategoryName());

   Util::SetupDateLabel(frm, IdDateSelectTrigger, _Record.getDateTime());
   Frm::SetText(frm, IdVendorField, _Record.getVendor());
   Frm::SetupPopupList(frm, IdGradePopTrigger, IdGradeList, _Record.getGrade());
   Char buff[16];
   Frm::SetText(frm, IdAmountField, NumberFormat::Format(buff, _Record.getAmount(), 2));
   Frm::SetText(frm, IdCostField, NumberFormat::Format(buff, _Record.getCost()));
   Frm::SetText(frm, IdNoteField, _Record.getNote());
   Fld::UpdateScrollbar(frm, IdNoteField, IdNoteFieldScrollbar);

   MemHandle strH;
   
   strH = ::DmGetResource(strRsc, IdKmString + _Record.getDistanceUnitType());
   Frm::SetText(frm, IdDistanceUnitField, (Char *)::MemHandleLock(strH));
   ::MemHandleUnlock(strH);
   ::DmReleaseResource(strH);
   strH = ::DmGetResource(strRsc, IdLiterString + _Record.getAmountUnitType());
   Frm::SetText(frm, IdAmountUnitField, (Char *)::MemHandleLock(strH));
   ::MemHandleUnlock(strH);
   ::DmReleaseResource(strH);

   DrawDistance(frm, _Record.getDistance());

   Frm::SetFocus(frm, IdVendorField);
}

void RecordForm::Open() {
   FormPtr frm = ::FrmGetActiveForm();

   Database *db = Mileage::GetDatabase();
   UInt16 index = db->getMarkedRecordIndex(); 
   MemHandle h = db->queryRecord(index);
   _Record.copy(::MemHandleLock(h));
   ::MemHandleUnlock(h);
   db->changeCategory(db->getRecordCategory(index));

   ::FrmDrawForm(frm);
   ::FrmUpdateForm(IdRefillForm, frmRedrawUpdateCode);
}

Boolean RecordForm::HandleCtlSelectEvent(EventPtr e) {
   FormPtr frm = ::FrmGetActiveForm();
   switch (e->data.ctlSelect.controlID) {
	  case IdCategoryPopTrigger:
		 Mileage::GetDatabase()->selectCategory(IdCategoryPopTrigger, IdCategoryList, false);
		 return true;

	  case IdDateSelectTrigger: {
		 DateTimeType dt = *(_Record.getDateTime());
		 MemHandle h = ::DmGetResource(strRsc, IdSelectDateString);
		 if (::SelectDay(::selectDayByDay, &dt.month, &dt.day, &dt.year, (char *)::MemHandleLock(h))) {
			_Record.setDateTime(&dt);
			Util::SetupDateLabel(frm, IdDateSelectTrigger, &dt);
		 }
		 ::MemHandleUnlock(h);
		 ::DmReleaseResource(h);
		 return true;
	  }

	  case IdDistanceButton1:
	  case IdDistanceButton2:
	  case IdDistanceButton3:
	  case IdDistanceButton4:
	  case IdDistanceButton5:
	  {
		 Int16 place = e->data.ctlSelect.controlID - IdDistanceButton5;
		 UInt32 target = 1;
		 for (Int16 i = 0; i < place; i++) {
			target *= 10;
		 }
		 UInt32 upper = target * 10;
		 UInt32 d = _Record.getDistance();
		 UInt32 fraction = d % upper;
		 d -= fraction;
		 fraction += target;
		 d += (fraction % upper);
		 _Record.setDistance(d);
		 DrawDistance(frm, d);
		 break;
	  }

	  case IdVendorPopTrigger: {
		 VendorHistory *history = Mileage::GetHistory();
		 ListPtr lst = (ListPtr)Frm::GetObjectPtr(frm, IdVendorList);
		 UInt16 num = history->count();
		 UInt16 min = (num == 0 ? 1 : num);
		 ::LstSetListChoices(lst, NULL, min);
		 ::LstSetHeight(lst, min);
		 ::LstSetDrawFunction(lst, DrawListLine);
		 short selected = ::LstPopupList(lst);
		 if (num > 0 && selected != -1 ) {
			Apply();
			_Record.setVendor((*history)[selected]);
			if (VendorHistory::IsAutoComplete()) {
			   Database *db = Mileage::GetDatabase();
			   if (!db->isNew() && ::FrmAlert(IdOverWriteRecordAlert) == 1) {
				  return true;
			   }
			   UInt32 uniqueId = history->getUniqueId(selected);
			   UInt16 index;
			   MemHandle h;
			   if (db->findRecordByID(uniqueId, &index) == 0 &&
				   (h = db->queryRecord(index)) != 0) {
				  DateTimeType dt = *_Record.getDateTime();
				  _Record.copy(::MemHandleLock(h));
				  ::MemHandleUnlock(h);
				  _Record.setDateTime(&dt);
				  _Record.setExported(false);
				  db->changeCategory(db->getRecordCategory(index));
			   } else {
				  ::FrmAlert(IdHistoryRecordDeletedAlert);
			   }
			}
			::FrmUpdateForm(IdRefillForm, frmRedrawUpdateCode);
		 }
		 return true;
	  }

	  case IdDoneButton:
		 ::FrmGotoForm(IdRecordListForm);
		 return true;

	  default:
		 break;
   }
   return false;
}

void RecordForm::Apply() {
   Char *buff = (Char *)::MemPtrNew(MAX_NOTE_CHARS);
   FormPtr frm = ::FrmGetActiveForm();
   _Record.setVendor(Frm::GetText(frm, IdVendorField, buff));
   _Record.setGrade((Record::GradeType)Frm::GetLstSelection(frm, IdGradeList));
   _Record.setAmount(NumberFormat::Parse(Frm::GetText(frm, IdAmountField, buff), 2));
   _Record.setCost(NumberFormat::Parse(Frm::GetText(frm, IdCostField, buff)));
   _Record.setNote(Frm::GetText(frm, IdNoteField, buff));
   ::MemPtrFree(buff);
}

void RecordForm::Save() {
   Database *db = Mileage::GetDatabase();
   if (db->getMarkedRecordIndex() == Database::NO_RECORD_SELECTED) {
	  // deleted.
	  return;
   }

   Apply();

   if (_Record.getAmount() == 0) {
	  db->deleteRecord();
   } else {
	  UInt16 index = db->updateRecord(_Record);
	  Char *vendor = _Record.getVendor();
	  if (vendor[0] != '\0') {
		 UInt32 uniqueId;
		 if (db->getUniqueId(index, &uniqueId) == 0) {
			Mileage::GetHistory()->push(vendor, uniqueId);
		 }
	  }
   }
}

Boolean RecordForm::HandleEvent(EventPtr e)
{
   switch (e->eType)
   {
	  case frmOpenEvent:
		 Open();
		 return true;

	  case frmCloseEvent:
		 Save();
		 break;

	  case frmUpdateEvent:
		 UpdateForm();
		 break;

	  case ctlSelectEvent:
		 return HandleCtlSelectEvent(e);
		 
	  case menuEvent:
		 if (!Frm::HandleCommonMenuItems(e->data.menu.itemID)) {
			switch (e->data.menu.itemID) {
			   case IdRecordExport:
				  Apply();
				  if (ExportForm::DoModal(_Record) == IdOKButton) {
					 _Record.setExported(true);
				  }
				  break;

			   case IdRecordDelete: {
				  if (DeleteRecordForm::DoModal() == IdOKButton) {
					 ::FrmGotoForm(IdRecordListForm);
				  }
				  break;
			   }

			   default:
				  break;
			}
		 }
		 break;

	  case fldChangedEvent:
		 Fld::UpdateScrollbar(::FrmGetActiveForm(), IdNoteField, IdNoteFieldScrollbar);
		 return true;

	  case sclRepeatEvent:
		 Fld::ScrollLines(IdNoteField, IdNoteFieldScrollbar,
						  e->data.sclRepeat.newValue - e->data.sclRepeat.value, false);
		 return false;

	  case keyDownEvent:
		 if (e->data.keyDown.chr == pageUpChr) {
			Fld::PageScroll(IdNoteField, IdNoteFieldScrollbar, winUp);
			return true;
		 } else if (e->data.keyDown.chr == pageDownChr) {
			Fld::PageScroll(IdNoteField, IdNoteFieldScrollbar, winDown);
			return true;
		 }
		 return false;

	  default:
		 break;
   }
   return false;
}
