#include <PalmOS.h>
#include <CatooCraft/Util.h>
#include <CatooCraft/FrmHelper.h>
#include <CatooCraft/SclHelper.h>
#include <CatooCraft/NumberFormat.h>
#include "RecordListForm.h"
#include "PrefsForm.h"
#include "AboutForm.h"
#include "Record.h"

using namespace CatooCraftLibrary;

void RecordListForm::DrawTableLine(void *tbl, Int16 row, Int16 column, RectanglePtr bounds) {
   Database *db = Mileage::GetDatabase();
   UInt16 index = ::TblGetRowID((TablePtr)tbl, row);
   MemHandle h = db->queryRecord(index);
   FontID prev;
   if (index == db->getMarkedRecordIndex())
	  prev = (FontID)::FntSetFont(::boldFont);
   else 
	  prev = (FontID)::FntSetFont(::stdFont);

   Record r(::MemHandleLock(h));
   DateTimePtr dtp = r.getDateTime();
   Util::DrawDate(dtp->month, dtp->day, bounds);
   RectangleType rec = *bounds;
   bounds->topLeft.x += 30;
   bounds->extent.x = 60;
   if (!r.isExported())
	  Util::DrawCharsToFitWidth("*", bounds);
   bounds->topLeft.x += 8;
   Util::DrawCharsToFitWidth(r.getVendor(), bounds);
   Char buff[16];
   Util::DrawCharsToFitWidth(r.getCalculation(buff), &rec, Util::RIGHT_ALIGN);
   ::MemHandleUnlock(h);

   ::FntSetFont(prev);
}

void RecordListForm::Open() {
   FormPtr frm = ::FrmGetActiveForm();
   ::FrmDrawForm(frm);

   TablePtr tbl = (TablePtr)Frm::GetObjectPtr(frm, IdListTable);
   ::TblSetColumnUsable(tbl, 0, true);
   ::TblSetCustomDrawProcedure(tbl, 0, RecordListForm::DrawTableLine);
   UInt16 numOfRows = ::TblGetNumberOfRows(tbl);
   for(UInt16 row = 0; row < numOfRows; row++) {
	  ::TblSetItemStyle(tbl, row, 0, customTableItem);
	  ::TblSetRowUsable(tbl, row, false);
   }

   Frm::SetupPopupList(frm, IdCalculationPopTrigger, IdCalculationList, Record::GetCalculationType());
   ::FrmDrawForm(frm);
   ::FrmUpdateForm(IdRecordListForm, frmRedrawUpdateCode);
}

void RecordListForm::UpdateForm() {
   FormPtr frm = ::FrmGetActiveForm();

   Database *db = Mileage::GetDatabase();
   ::CtlSetLabel(Frm::GetObjectPtr(frm, IdCategoryPopTrigger), db->getCategoryName());

   UpdateTable(0);
}

void RecordListForm::UpdateTable(Int16 pos) {
   Database *db = Mileage::GetDatabase();
   FormPtr frm = ::FrmGetActiveForm();
   TablePtr tbl = (TablePtr)Frm::GetObjectPtr(frm, IdListTable);
   ScrollBarPtr bar = (ScrollBarPtr)Frm::GetObjectPtr(frm, IdListScrollbar);

   UInt16 numOfRows = ::TblGetNumberOfRows(tbl);
   UInt16 maxValue = db->numRecords();

   if (maxValue > numOfRows)
	  maxValue -= numOfRows;
   else
	  maxValue = 0;

   ::SclSetScrollBar(bar, pos, 0, maxValue, numOfRows);
   ::SclDrawScrollBar(bar);

   UInt16 index = 0;
   db->seekRecord(&index, pos, dmSeekForward);
   for(UInt16 row = 0; row < numOfRows; row++) {
	  MemHandle h = db->queryNext(&index);
	  if (h != NULL) {
		 ::TblSetRowUsable(tbl, row, true);
		 ::TblSetRowID(tbl, row, index);
		 ::TblMarkRowInvalid(tbl, row);
		 index++;
	  } else {
		 ::TblSetRowUsable(tbl, row, false);
	  }
   }

   ::TblEraseTable(tbl);
   ::TblDrawTable(tbl);
}

void RecordListForm::ScrollTable(WinDirectionType direction) {
   FormPtr frm = ::FrmGetActiveForm();
   ScrollBarPtr bar = (ScrollBarPtr)Frm::GetObjectPtr(frm, IdListScrollbar);
   Boolean changed;
   Int16 newValue = Scl::GetNewValue(bar, direction, &changed);
   if (changed)
	  UpdateTable(newValue);
}

Boolean RecordListForm::HandleCtlSelectEvent(EventPtr e) {
   Database *db = Mileage::GetDatabase();
   switch (e->data.ctlSelect.controlID) {
	  case IdCategoryPopTrigger:
		 db->selectCategory(IdCategoryPopTrigger, IdCategoryList);
		 UpdateTable(0);
		 return true;

	  case IdRefillButton:
		 db->createRecord();
		 ::FrmGotoForm(IdRefillForm);
		 return true;

	  default:
		 break;
   }
   return false;
}

Boolean RecordListForm::HandleMenuEvent(EventPtr e) {
   switch (e->data.menu.itemID) {
	  case IdOptionsPreferences:
		 if (PrefsForm::DoModal() == IdOKButton)
			::FrmUpdateForm(IdRecordListForm, frmRedrawUpdateCode);
		 return true;

	  case IdOptionsAbout:
		 AboutForm::DoModal();
		 return true;

	  default:
		 break;
   }
   return false;
}

Boolean RecordListForm::HandleEvent(EventPtr e)
{
   switch (e->eType)
   {
	  case frmOpenEvent:
		 Open();
		 return true;

	  case ctlSelectEvent:  
		 return HandleCtlSelectEvent(e);

	  case popSelectEvent:
		 if (e->data.popSelect.listID == IdCalculationList) {
			Record::SetCalculationType((Record::CalculationType)e->data.popSelect.selection);
			::FrmUpdateForm(IdRecordListForm, frmRedrawUpdateCode);
		 }
		 break;

	  case tblSelectEvent:
	  {
		 UInt16 index = ::TblGetRowID((TablePtr)e->data.tblSelect.pTable,
									  e->data.tblSelect.row);
		 Mileage::GetDatabase()->markRecord(index);
		 ::FrmGotoForm(IdRefillForm);
		 return true;
	  }

	  case menuEvent:
		 return HandleMenuEvent(e);

	  case frmUpdateEvent:
		 UpdateForm();
		 break;

	  case sclRepeatEvent:
		 UpdateTable(e->data.sclRepeat.newValue);
		 break;

	  case keyDownEvent:
		 if (e->data.keyDown.chr == pageUpChr) {
			ScrollTable(winUp);
			return true;
		 } else if (e->data.keyDown.chr == pageDownChr) {
			ScrollTable(winDown);
			return true;
		 }
		 break;

	  default:
		 break;
   }
   return false;
}
