#include <TestSuite.h>
#include <TestCaller.h>
#include "RecordTest.h"

Test* RecordTest::suite() {
    TestSuite* suite = new TestSuite("RecordTest");
	suite->addTest(new TestCaller<RecordTest>("setter getter", &RecordTest::testSetterGetter));
	suite->addTest(new TestCaller<RecordTest>("pack unpack", &RecordTest::testPackUnpack));
	suite->addTest(new TestCaller<RecordTest>("calculation", &RecordTest::testCalculation));

    return suite;
}

// setUp \bh͊eeXg\bh (testPackUnpack Ȃ) Ă΂܂ł
// т郁\bhłDŃeXĝ߂ɕKvȎOȂǂs܂D
void RecordTest::setUp() {
   // eXgJnO _freeBytes Ɍ݂̗eʂۑĂ
   if (::MemCardInfo(0, 0, 0, 0, 0, 0, 0, &_freeBytes) != 0)
	  Fail("MemCardInfo Ɏs");
}

// tearDown \bh͊eeXg\bhĂ΂ꂽɌĂяo郁\bhłD
// ŃeXǧnȂǂs܂D
void RecordTest::tearDown() {
   // eXgIɕۑĂ _freeBytes ƔrĒlقȂĂ
   // ǂŃ[NĂ
   UInt32 after;
   if (::MemCardInfo(0, 0, 0, 0, 0, 0, 0, &after) != 0)
	  Fail("MemCardInfo Ɏs");
   AssertEquals((long)(_freeBytes & 0x7fff), (long)(after & 0x7fff));
}

void RecordTest::setupRecord(Record &r) {
   DateTimeType dt;
   dt.year = 2001;
   dt.month = 2;
   dt.day = 17;

   r.setDateTime(&dt);
   r.setDistance(300);
   r.setDistanceUnitType(Record::MILE);
   r.setVendor("~X^h");
   r.setGrade(Record::REGULAR);
   r.setAmount(3133);
   r.setAmountUnitType(Record::GALLON_UK);
   r.setCost(3412);
   r.setNote("߂߂");
}

void RecordTest::testSetterGetter() {
   Record r;
   setupRecord(r);

   DateTimeType *dtp = r.getDateTime();
   AssertEquals(2001, dtp->year);
   AssertEquals(2, dtp->month);
   AssertEquals(17, dtp->day);
   AssertEquals(300, r.getDistance());
   AssertEquals(Record::MILE, r.getDistanceUnitType());
   AssertEquals("~X^h", r.getVendor());
   AssertEquals(Record::REGULAR, r.getGrade());
   AssertEquals(3133, r.getAmount());
   AssertEquals(Record::GALLON_UK, r.getAmountUnitType());
   AssertEquals(3412, r.getCost());
   AssertEquals("߂߂", r.getNote());
}

void RecordTest::assertEqualsRecord(Record &expected, Record &actual) {
   DateTimeType *dtexpected = expected.getDateTime();
   DateTimeType *dtactual = actual.getDateTime();
   AssertEquals(dtexpected->year, dtactual->year);
   AssertEquals(dtexpected->month, dtactual->month);
   AssertEquals(dtexpected->day, dtactual->day);

   AssertEquals(expected.getDistance(), actual.getDistance());
   AssertEquals(expected.getDistanceUnitType(), actual.getDistanceUnitType());
   AssertEquals(expected.getVendor(), actual.getVendor());
   AssertEquals(expected.getGrade(), actual.getGrade());
   AssertEquals(expected.getAmount(), actual.getAmount());
   AssertEquals(expected.getAmountUnitType(), actual.getAmountUnitType());
   AssertEquals(expected.getCost(), actual.getCost());
   AssertEquals(expected.getNote(), actual.getNote());
}

void RecordTest::testPackUnpack() {
   Record r, rr;
   setupRecord(r);

   DmOpenRef db = ::DmOpenDatabaseByTypeCreator('DATA', 'memo', dmModeReadWrite);
   Assert(db != NULL);

   MemHandle h = ::DmNewHandle(db, r.size());
   MemPtr p = ::MemHandleLock(h);
   r.pack(p);
   rr.copy(p);
   Record rrr(p);

   // RXgN^ p n̂͂̃nhbNĂ
   // ԂANZXł
   assertEqualsRecord(r, rr);
   assertEqualsRecord(r, rrr);

   ::MemHandleUnlock(h);
   ::MemHandleFree(h);
   ::DmCloseDatabase(db);

   // copy ̂̓f[^x[XɊ֌WȂANZXł
   assertEqualsRecord(r, rr);
}

void RecordTest::testCalculation() {
   Char buff[32];
   Record::SetDistanceUnitType(Record::KM);
   Record::SetAmountUnitType(Record::LITER);

   Record r;
   r.setDistance(3000);
   r.setAmount(3133);
   r.setCost(3412);

   Record::SetCalculationType(Record::CALC_DISTANCE);
   AssertEquals("300.0", r.getCalculation(buff));	// 300.0 km
   Record::SetCalculationType(Record::CALC_AMOUNT);
   AssertEquals("31.33", r.getCalculation(buff)); // 31.33 liter
   Record::SetCalculationType(Record::CALC_COST);
   AssertEquals("3,412", r.getCalculation(buff)); // 3,412 yen
   Record::SetCalculationType(Record::CALC_MILEAGE);
   AssertEquals("9.57", r.getCalculation(buff)); // 9.57 km/liter
   Record::SetCalculationType(Record::CALC_PRICE);
   AssertEquals("108", r.getCalculation(buff));	// 108 yen/liter

   Record::SetDistanceUnitType(Record::MILE);
   Record::SetAmountUnitType(Record::GALLON_US);
   Record::SetCalculationType(Record::CALC_DISTANCE);
   AssertEquals("186.3", r.getCalculation(buff));	// 300.0 km / 1.61 = 186.3 mile
   Record::SetCalculationType(Record::CALC_AMOUNT);
   AssertEquals("8.26", r.getCalculation(buff)); // 31.33 liter / 3.79 = 8.26
   Record::SetCalculationType(Record::CALC_COST);
   AssertEquals("3,412", r.getCalculation(buff)); // 3,412 yen
   Record::SetCalculationType(Record::CALC_MILEAGE);
   AssertEquals("22.55", r.getCalculation(buff)); // 22.51 mile/gallon(US)
   Record::SetCalculationType(Record::CALC_PRICE);
   AssertEquals("412", r.getCalculation(buff));	// 412 yen/mile
}
