#include <TestSuite.h>
#include <TestCaller.h>
#include "VendorHistoryTest.h"

Test* VendorHistoryTest::suite() {
    TestSuite* suite = new TestSuite("VendorHistoryTest");
	suite->addTest(new TestCaller<VendorHistoryTest>("setup", &VendorHistoryTest::testSetup));
	suite->addTest(new TestCaller<VendorHistoryTest>("push", &VendorHistoryTest::testPush));

    return suite;
}

void VendorHistoryTest::setUp() {
   if (::MemCardInfo(0, 0, 0, 0, 0, 0, 0, &_freeBytes) != 0)
	  Fail("MemCardInfo Ɏs");
}

void VendorHistoryTest::tearDown() {
   UInt32 after;
   if (::MemCardInfo(0, 0, 0, 0, 0, 0, 0, &after) != 0)
	  Fail("MemCardInfo Ɏs");
   AssertEquals((long)(_freeBytes & 0x7fff), (long)(after & 0x7fff));
}

UInt16 VendorHistoryTest::setHistoryData(Char *p, Char *name, UInt32 uniqueId) {
   UInt16 size;
   size = ::StrLen(name) + 1;
   ::StrCopy(p, name);
   p += size;
   *(p++) = (uniqueId >> (8*3));
   *(p++) = (uniqueId >> (8*2)) & 0x00ff;
   *(p++) = (uniqueId >> (8*1)) & 0x00ff;
   *(p++) = uniqueId & 0x00ff;
   return size + 4;
}

void VendorHistoryTest::testSetup() {
   Char buff[256];
   Char *p = buff;
   p += setHistoryData(p, "top", 10);
   p += setHistoryData(p, "second", 20);
   setHistoryData(p, "third", 15);

   VendorHistory history;
   history.setup(buff, 3);
   ::MemSet(buff, 256, 0);
   AssertEquals(3, history.count());
   AssertEquals("top", history[0]);
   AssertEquals("second", history[1]);
   AssertEquals("third", history[2]);
   AssertEquals(4+4 + 7+4 + 6+4, history.size());
   history.pack(buff);
   AssertEquals("top", &buff[0]);
   AssertEquals("second", &buff[4+4]);
   AssertEquals("third", &buff[4+4 + 7+4]);

   AssertEquals(10, history.getUniqueId(0));
   AssertEquals(20, history.getUniqueId(1));
   AssertEquals(15, history.getUniqueId(2));
}

void VendorHistoryTest::testPush() {
   VendorHistory history;
   history.push("first", 10);
   history.push("second", 20);
   history.push("third", 15);
   AssertEquals(3, history.count());
   AssertEquals("third", history[0]);
   AssertEquals("second", history[1]);
   AssertEquals("first", history[2]);

   history.push("second", 100);
   AssertEquals(3, history.count());
   AssertEquals("second", history[0]);
   AssertEquals("third", history[1]);
   AssertEquals("first", history[2]);

   history.push("a1", 10001);
   history.push("a2", 10002);
   history.push("a3", 10003);
   history.push("a4", 10004);
   history.push("a5", 10004);
   history.push("a6", 10006);
   history.push("a7", 10007);
   history.push("a8", 10008);
   history.push("a9", 10009);
   history.push("a10", 10010);

   AssertEquals("a10", history[0]);
   AssertEquals("a9", history[1]);
   AssertEquals("a8", history[2]);
   AssertEquals("a7", history[3]);
   AssertEquals("a6", history[4]);
   AssertEquals("a5", history[5]);
   AssertEquals("a4", history[6]);
   AssertEquals("a3", history[7]);
   AssertEquals("a2", history[8]);
   AssertEquals("a1", history[9]);
}
