#include "PalmTestResult.h"

void PalmTestResult::setText(FieldPtr fld, const Char *str) {
   MemHandle txtH;
	
   txtH = ::MemHandleNew(StrLen(str) + 1);
   ErrFatalDisplayIf(!txtH, "mem error");
		
   ::StrCopy((char *)::MemHandleLock(txtH), str);
   ::MemHandleUnlock(txtH);
   MemHandle oldTxtH = (MemHandle)::FldGetTextHandle(fld);
   ::FldSetTextHandle(fld, (MemHandle)txtH);
   if (oldTxtH)
	  ::MemHandleFree(oldTxtH);
}

void PalmTestResult::setNumber(FieldPtr fld, const Int32 value) {
   Char buff[16];
   ::StrPrintF(buff, "%d", (int)value);
   setText(fld, buff);
}

void PalmTestResult::startTest(Test *test)
{
   TestResult::startTest(test);
   setNumber(_runs, runTests());
   setNumber(_failures, 0);
   setNumber(_errors, 0);
   ::FldDrawField(_runs);
   ::FldDrawField(_failures);
   ::FldDrawField(_errors);
   _sw.startStop(::TimGetTicks());
}

void PalmTestResult::endTest(Test *test)
{
   UInt8 pattern[] = {
	  0xaa, 0xaa, 0xaa, 0xaa,
	  0xaa, 0xaa, 0xaa, 0xaa
   };
   ::WinSetPattern((CustomPatternType *)pattern);

   _sw.startStop(::TimGetTicks());
   TestResult::endTest(test);
   setText(_finished, _sw.elapsedTime());
   ::FldDrawField(_finished);
   int tmp = _rectangle->extent.x;
   _rectangle->extent.x = (int)_rectangle->extent.x * runTests() / _count;
   ::WinFillRectangle(_rectangle, 0);
   _rectangle->extent.x = tmp;
}

void PalmTestResult::addFailure(Test *test, Exception *e)
{
   TestResult::addFailure(test, e);
   _sw.startStop(::TimGetTicks());
   setNumber(_failures, testFailures());
   ::FldDrawField(_failures);
   _sw.startStop(::TimGetTicks());
}

void PalmTestResult::addError(Test *test, Exception *e)
{
   TestResult::addError(test, e);
   _sw.startStop(::TimGetTicks());
   setNumber(_errors, testErrors());
   ::FldDrawField(_errors);
   _sw.startStop(::TimGetTicks());
}
