#include "PalmUnit.h"
#include "PalmUnitRsc.h"

//#include "SampleTest.h"
//#include "SampleTest2.h"
#include "../AllTests.h"

//static SampleTest testSample1("Sample Test");
//static SampleTest2 testSample2("Sample Test 2");

PalmTestResult * PalmUnit::_Result = NULL;
TestRunner PalmUnit::_Runner;
RectangleType PalmUnit::_Rectangle;

ControlPtr PalmUnit::GetObjectPtr(FormPtr frm, UInt16 objectId) {
   return (ControlPtr)::FrmGetObjectPtr(frm, ::FrmGetObjectIndex(frm, objectId));
}

void PalmUnit::DrawTable(void *tbl, Int16 row, Int16 column, RectanglePtr bounds)
{
   Boolean	fits;
   TestFailure * failure = (TestFailure *)::TblGetRowData((TablePtr)tbl, row);
   Int16 lstWidth = bounds->extent.x;
   Int16 textLen = ::StrLen(failure->toString());
   ::FntCharsInWidth(failure->toString(), &lstWidth, &textLen, &fits);
   ::WinDrawChars(failure->toString(), textLen, bounds->topLeft.x, bounds->topLeft.y);
}

void PalmUnit::UpdateTable(FormPtr frm, PalmTestResult * result, UInt16 pos)
{
   static UInt16 previousPos = 0;
   if (pos == (UInt16)-1)
	  pos = previousPos;
   else
	  previousPos = pos;

   TablePtr tbl = (TablePtr)GetObjectPtr(frm, IdFailuresTable);
   ScrollBarPtr bar = (ScrollBarPtr)GetObjectPtr(frm, IdFailuresTableScrollbar);
   ::TblSetColumnUsable(tbl, 0, true);
   ::TblSetCustomDrawProcedure(tbl, 0, DrawTable);
   vector<TestFailure *>& failures = result->failures();
   vector<TestFailure *>& errors = result->errors();

   vector<TestFailure *> failuresAndErrors;
   for (vector<TestFailure *>::iterator it = failures.begin();
		it != failures.end();
		it++) {
	  failuresAndErrors.push_back(*it);
   }
   for (vector<TestFailure *>::iterator it = errors.begin();
		it != errors.end();
		it++) {
	  failuresAndErrors.push_back(*it);
   }
   
   vector<TestFailure *>::iterator it = failuresAndErrors.begin();
   for (UInt16 i = 0; i<pos; i++)
	  it++;

   Int16 rows = ::TblGetNumberOfRows(tbl);
   for(Int16 row = 0; row < rows; row++)
   {
	  if (it != failuresAndErrors.end())
	  {
		 ::TblSetItemStyle(tbl, row, 0, customTableItem);
		 ::TblSetRowUsable(tbl, row, true);
		 ::TblSetRowData(tbl, row, (UInt32)(*it));
		 ::TblMarkRowInvalid(tbl, row);
		 it++;
	  }
	  else
	  {
		 ::TblSetRowUsable(tbl, row, false);
	  }
   }

   UInt16 maxValue;
   if (failuresAndErrors.size() > rows)
	  maxValue = failuresAndErrors.size() - rows;
   else
	  maxValue = 0;

   ::SclSetScrollBar(bar, pos, 0, maxValue, rows);
   ::TblEraseTable(tbl);
   ::TblDrawTable(tbl);
   ::SclDrawScrollBar(bar);
}

void PalmUnit::HandleTableSelect(EventPtr e)
{
   TestFailure * failure = (TestFailure *)::TblGetRowData((TablePtr)e->data.tblSelect.pTable,
														  e->data.tblSelect.row);
   ::FrmCustomAlert(IdFailureAlert, failure->toString(), NULL, NULL);
}

Boolean PalmUnit::MainFormHandleEvent(EventPtr e) {
   Boolean handled = false;
   FormPtr frm = ::FrmGetActiveForm();

   switch (e->eType) {
	  case frmOpenEvent:
	  {
		 ::FrmDrawForm(frm);
		 _Rectangle.topLeft.x = 10;
		 _Rectangle.topLeft.y = 30;
		 _Rectangle.extent.x = 140;
		 _Rectangle.extent.y = 10;
		 ::WinDrawRectangleFrame(simpleFrame, &_Rectangle);

		 _Runner.addTest("All Tests", new AllTests);
//		 _Runner.addTest("Sample Test 1", testSample1.suite());
//		 _Runner.addTest("Sample Test 2", testSample2.suite());

		 {
			int i = 0;
			Char* * itemsText;
			mappings &tests = _Runner.tests();
			itemsText = new Char*[tests.size()];
			for (mappings::iterator it = tests.begin();
				 it != tests.end();
				 it++)
			{
			   itemsText[i++] = (*it).first;
			}
			::LstSetListChoices((ListPtr)GetObjectPtr(frm, IdTestsListList),
								itemsText, tests.size());
			::LstSetHeight((ListPtr)GetObjectPtr(frm, IdTestsListList),
						   tests.size() > 10 ? 10 : tests.size());
			::LstSetSelection((ListPtr)GetObjectPtr(frm, IdTestsListList), (UInt16)-1);
		 }
		 handled = true;
		 break;
	  }

	  case ctlSelectEvent:
	  {
		 switch(e->data.ctlSelect.controlID)
		 {
			case IdRunButton:
			{
			   ::WinEraseRectangle(&_Rectangle, 0);
			   ::WinDrawRectangleFrame(simpleFrame, &_Rectangle);

			   if (_Result) delete _Result;
			   _Result = new PalmTestResult;
			   _Result->setRunsField((FieldPtr)GetObjectPtr(frm, IdRunsField));
			   _Result->setErrorsField((FieldPtr)GetObjectPtr(frm, IdErrorsField));
			   _Result->setFailuresField((FieldPtr)GetObjectPtr(frm, IdFailuresField));
			   _Result->setFinishedField((FieldPtr)GetObjectPtr(frm, IdFinishedField));
			   _Result->setProgressRectangle(&_Rectangle);

			   {
				  ListPtr lst = (ListPtr)GetObjectPtr(frm, IdTestsListList);
				  int selection = ::LstGetSelection(lst);
				  if (selection != -1)
				  {
					 _Runner.run(::LstGetSelectionText(lst, selection), _Result);
					 UpdateTable(frm, _Result, 0);
				  }
			   }
			   handled = true;
			   break;
			}
			default:
			   break;
		 }
		 break;
	  }

	  case tblSelectEvent:
		 HandleTableSelect(e);
		 UpdateTable(frm, _Result, (UInt16)-1);
		 break;

	  case sclRepeatEvent:
		 UpdateTable(frm, _Result, e->data.sclRepeat.newValue);
		 break;

	  case menuEvent:
		 switch(e->data.menu.itemID)
		 {
			case IdOptionsAbout:
			{
			   FormPtr frm = ::FrmInitForm(IdAboutForm);
			   ::FrmDoDialog(frm);
			   ::FrmDeleteForm(frm);
			   break;
			}
		 }
		 handled = true;
		 break;

	  default:
		 break;
   }

   return handled;
}

Boolean PalmUnit::HandleEvent(EventPtr e)
{
   FormPtr frm;
   UInt16    formId;
   Boolean handled = false;

   if (e->eType == frmLoadEvent) {
	  formId = e->data.frmLoad.formID;
	  frm = ::FrmInitForm(formId);
	  ::FrmSetActiveForm(frm);

	  switch(formId) {
		 case IdMainForm:
			::FrmSetEventHandler(frm, MainFormHandleEvent);
			break;
	  }
	  handled = true;
   }

   return handled;
}

UInt16 PalmUnit::StartApplication(void)
{
   ::FrmGotoForm(IdMainForm);
   return 0;
}

void PalmUnit::StopApplication(void)
{
   if (_Result) delete _Result;
   ::FrmSaveAllForms();
   ::FrmCloseAllForms();
}

void PalmUnit::EventLoop(void)
{
   UInt16 err;
   EventType e;

   do {
	  ::EvtGetEvent(&e, evtWaitForever);
	  if (! ::SysHandleEvent(&e))
		 if (! ::MenuHandleEvent(NULL, &e, &err))
			if (! HandleEvent(&e))
			   ::FrmDispatchEvent (&e);
   } while (e.eType != appStopEvent);
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
   UInt16 err;

   if (cmd == sysAppLaunchCmdNormalLaunch) {
	  err = PalmUnit::StartApplication();
	  if (err)
		 return err;

	  PalmUnit::EventLoop();
	  PalmUnit::StopApplication();
   } else {
	  return sysErrParamErr;
   }
   return 0;
}
